library(lme4)  # load library
library(arm) # convenience functions for regression in R
library(car) ## for normality check
library(MASS) # for normality check
library("mlmRev") ## for gamma dist
library("AICcmodavg")
library("optimx")

filename="/Users/kprovost/Dropbox (AMNH)/Cardinalis MS/FOR_DRYAD/data/playback_results/combined_both_years_data_all_columns_for_pub.csv"
card = read.csv(filename)
names(card)
work = card[card$PointWork==1,]

chi = card[card$Desert=="CHI",]
chi_work = work[work$Desert=="CHI",]

son = card[card$Desert=="SON",]
son_work = work[work$Desert=="SON",]

## FIRST DO FOR CHIHUAHUAN ONLY

## gamma distributions require non-negative data, so
## adjust the PC1 values so that they are non-negative
PC1.NONZERO = chi$PC1.CHI * -1 +1
PC1.NONWORK = chi_work$PC1.CHI * -1 +1

PRE.NONZERO = chi$PRE.PC1.CHI * -1 +1
PRE.NONWORK = chi_work$PRE.PC1.CHI * -1 +1

## generate functions to calculate R-squared, adjusted R-squared
manualRsq = function(model,actual) {
  pred = predict(model)
  resid = resid(model,type="working",scaled=FALSE)
  minMean = actual-mean(actual)
  mmSq = minMean^2
  ssTOT = sum(mmSq)
  residSq = resid^2
  ssRES = sum(residSq)
  r2 = 1 - (ssRES/ssTOT)
  return(r2)
}

adjRsqManual = function(Rsq,datapoints,predictors){
  ## 1 - ((1 - Rsq)*((datapoints-1)/(datapoints-predictors-1)))
  onemin = 1 - Rsq
  num = datapoints-1
  den = datapoints-predictors-1
  adj = 1 - ((onemin)*(num/den))
  return(adj)
}

## test two optimization algorithms on the Treatment-Included dataset
## model in manuscript:
## Aggression(PC1) ~ Treatment + Pre-Treatment Aggression + Site(Random Effect) + Stimulus Set(Random Effect)
## equivalent model below:
## PC1.NONZERO ~ Type + PRE.NONZERO + (1 | Point) + (1 | Song)

## model 1A: full dataset, with treatment, L-BFGS-B model
chi_full_withTreat_lbfgsb = glmer(PC1.NONZERO ~ Type + PRE.NONZERO + (1 | Point) + (1 | Song),
                              data = chi, family = Gamma(link = "log"),
                              control = glmerControl(optimizer = "optimx", calc.derivs = FALSE,
                                                     optCtrl = list(method = "L-BFGS-B", starttests = FALSE, kkt = FALSE)))  
## model 1B: full dataset, no treatment, L-BFGS-B model
chi_full_noTreat_lbfgsb = glmer(PC1.NONZERO ~ PRE.NONZERO + (1 | Point) + (1 | Song),
                              data = chi, family = Gamma(link = "log"),
                              control = glmerControl(optimizer = "optimx", calc.derivs = FALSE,
                                                     optCtrl = list(method = "L-BFGS-B", starttests = FALSE, kkt = FALSE)))  

## model 2A: full dataset, with treatment, BOBYQA model
chi_full_withTreat_bobyqa = glmer(PC1.NONZERO ~ Type + PRE.NONZERO + (1 | Point) + (1 | Song),
                              data = chi, family = Gamma(link = "log"),
                              control = glmerControl(optimizer = "optimx", calc.derivs = FALSE,
                                                     optCtrl = list(method = "bobyqa", starttests = FALSE, kkt = FALSE)))  
## model 2B: full dataset, no treatment, BOBYQA model
chi_full_noTreat_bobyqa = glmer(PC1.NONZERO ~ PRE.NONZERO + (1 | Point) + (1 | Song),
                              data = chi, family = Gamma(link = "log"),
                              control = glmerControl(optimizer = "optimx", calc.derivs = FALSE,
                                                     optCtrl = list(method = "bobyqa", starttests = FALSE, kkt = FALSE)))  

## model 3A: detections-only dataset, with treatment, L-BFGS-B model
chi_work_withTreat_lbfgsb = glmer(PC1.NONWORK ~ Type + PRE.NONWORK + (1 | Point) + (1 | Song),
                              data = chi_work, family = Gamma(link = "log"),
                              control = glmerControl(optimizer = "optimx", calc.derivs = FALSE,
                                                     optCtrl = list(method = "L-BFGS-B", starttests = FALSE, kkt = FALSE)))  
## model 3B: detections-only dataset, no treatment, L-BFGS-B model
chi_work_noTreat_lbfgsb = glmer(PC1.NONWORK ~ PRE.NONWORK + (1 | Point) + (1 | Song),
                              data = chi_work, family = Gamma(link = "log"),
                              control = glmerControl(optimizer = "optimx", calc.derivs = FALSE,
                                                     optCtrl = list(method = "L-BFGS-B", starttests = FALSE, kkt = FALSE)))  

## model 4A: detections-only dataset, with treatment, BOBYQA model
chi_work_withTreat_bobyqa = glmer(PC1.NONWORK ~ Type + PRE.NONWORK + (1 | Point) + (1 | Song),
                              data = chi_work, family = Gamma(link = "log"),
                              control = glmerControl(optimizer = "optimx", calc.derivs = FALSE,
                                                     optCtrl = list(method = "bobyqa", starttests = FALSE, kkt = FALSE)))  
## model 4B: detections-only dataset, with treatment, BOBYQA model
chi_work_noTreat_bobyqa = glmer(PC1.NONWORK ~ PRE.NONWORK + (1 | Point) + (1 | Song),
                              data = chi_work, family = Gamma(link = "log"),
                              control = glmerControl(optimizer = "optimx", calc.derivs = FALSE,
                                                     optCtrl = list(method = "bobyqa", starttests = FALSE, kkt = FALSE)))  

## get AICc of each model 
AICc(chi_full_withTreat_lbfgsb) #318.44
AICc(chi_full_noTreat_lbfgsb) #319.6645
AICc(chi_full_withTreat_bobyqa) #318.44
AICc(chi_full_noTreat_bobyqa) #319.6647

AICc(chi_work_withTreat_lbfgsb) #348.0838
AICc(chi_work_noTreat_lbfgsb) #352.8693
AICc(chi_work_withTreat_bobyqa) #348.0838
AICc(chi_work_noTreat_bobyqa) #352.8693

## Note: L-BFGS-B model and BOBYQA model produce identical results, so 
## only use BOBYQA from now on (model 2 and model 4)

summary(chi_work_withTreat_bobyqa) {
"Generalized linear mixed model fit by maximum likelihood (Laplace Approximation) ['glmerMod']
 Family: Gamma  ( log )
Formula: PC1.NONWORK ~ Type + PRE.NONWORK + (1 | Point) + (1 | Song)
Data: chi_work
Control: glmerControl(optimizer = "optimx", calc.derivs = FALSE, optCtrl = list(method = "bobyqa",  
starttests = FALSE, kkt = FALSE))

AIC      BIC   logLik deviance df.resid 
347.0    370.8   -165.5    331.0      136 

Scaled residuals: 
Min      1Q  Median      3Q     Max 
-0.8292 -0.6980 -0.4571  0.2500  4.3307 

Random effects:
Groups   Name        Variance Std.Dev.
Point    (Intercept) 0.00000  0.0000  
Song     (Intercept) 0.07435  0.2727  
Residual             1.25685  1.1211  
Number of obs: 144, groups:  Point, 36; Song, 16

Fixed effects:
Estimate Std. Error t value Pr(>|z|)    
(Intercept)  0.27538    0.23321   1.181   0.2377    
TypeCW.C    -1.60225    0.39984  -4.007 6.14e-05 ***
TypePT.C    -0.70988    0.32062  -2.214   0.0268 *  
TypeWV      -0.03712    0.31746  -0.117   0.9069    
PRE.NONWORK  0.71247    0.12271   5.806 6.40e-09 ***
---
Signif. codes:  0 ‘***’ 0.001 ‘**’ 0.01 ‘*’ 0.05 ‘.’ 0.1 ‘ ’ 1

Correlation of Fixed Effects:
(Intr) TyCW.C TyPT.C TypeWV
TypeCW.C    -0.530                     
TypePT.C    -0.679  0.392              
TypeWV      -0.676  0.398  0.495       
PRE.NONWORK -0.272 -0.038  0.018 -0.017"}

summary(chi_work_withTreat_bobyqa) {
"Generalized linear mixed model fit by maximum likelihood (Laplace Approximation) ['glmerMod']
Family: Gamma  ( log )
Formula: PC1.NONWORK ~ Type + PRE.NONWORK + (1 | Point) + (1 | Song)
Data: chi_work
Control: glmerControl(optimizer = "optimx", calc.derivs = FALSE, optCtrl = list(method = "bobyqa",  
                                                                                starttests = FALSE, kkt = FALSE))

AIC      BIC   logLik deviance df.resid 
347.0    370.8   -165.5    331.0      136 

Scaled residuals: 
  Min      1Q  Median      3Q     Max 
-0.8292 -0.6980 -0.4571  0.2500  4.3307 

Random effects:
  Groups   Name        Variance Std.Dev.
Point    (Intercept) 0.00000  0.0000  
Song     (Intercept) 0.07435  0.2727  
Residual             1.25685  1.1211  
Number of obs: 144, groups:  Point, 36; Song, 16

Fixed effects:
  Estimate Std. Error t value Pr(>|z|)    
(Intercept)  0.27538    0.23321   1.181   0.2377    
TypeCW.C    -1.60225    0.39984  -4.007 6.14e-05 ***
  TypePT.C    -0.70988    0.32062  -2.214   0.0268 *  
  TypeWV      -0.03712    0.31746  -0.117   0.9069    
PRE.NONWORK  0.71247    0.12271   5.806 6.40e-09 ***
  ---
  Signif. codes:  0 ‘***’ 0.001 ‘**’ 0.01 ‘*’ 0.05 ‘.’ 0.1 ‘ ’ 1

Correlation of Fixed Effects:
  (Intr) TyCW.C TyPT.C TypeWV
TypeCW.C    -0.530                     
TypePT.C    -0.679  0.392              
TypeWV      -0.676  0.398  0.495       
PRE.NONWORK -0.272 -0.038  0.018 -0.017"
}

## perform an ANOVA on the full models 
Anova(chi_full_withTreat_bobyqa) {
'#Analysis of Deviance Table (Type II Wald chisquare tests)
#
#Response: PC1.NONZERO
#Chisq Df Pr(>Chisq)    
#Type         9.2231  3    0.02647 *  
#PRE.NONZERO 55.8636  1  7.768e-14 ***
#---
#Signif. codes:  0 ‘***’ 0.001 ‘**’ 0.01 ‘*’ 0.05 ‘.’ 0.1 ‘ ’ 1' }

Anova(chi_work_withTreat_bobyqa) {
'#Analysis of Deviance Table (Type II Wald chisquare tests)
#
#Response: PC1.NONWORK
#Chisq Df Pr(>Chisq)    
#Type        20.797  3   0.000116 ***
#PRE.NONWORK 33.710  1  6.398e-09 ***
#---
#Signif. codes:  0 ‘***’ 0.001 ‘**’ 0.01 ‘*’ 0.05 ‘.’ 0.1 ‘ ’ 1' }

## calculate R-squared and adjusted R-squared for those models 

rsq_chi_full_withTreat_bobyqa = (manualRsq(chi_full_withTreat_bobyqa,
                                           PC1.NONZERO)) ## 0.639
rsq_chi_full_noTreat_bobyqa = (manualRsq(chi_full_noTreat_bobyqa,
                                           PC1.NONZERO)) ## 0.642

rsq_chi_work_withTreat_bobyqa = (manualRsq(chi_work_withTreat_bobyqa,
                                           PC1.NONWORK)) ## 0.668
rsq_chi_work_noTreat_bobyqa = (manualRsq(chi_work_noTreat_bobyqa,
                                           PC1.NONWORK)) ## 0.709

## 4 parameters in the  model with treatment
adjRsqManual(rsq_chi_full_withTreat_bobyqa, # 0.633
             length(PC1.NONZERO),4) 
## 3 parameters in the model without treatment
adjRsqManual(rsq_chi_full_noTreat_bobyqa, # 0.637
             length(PC1.NONZERO),3) 

## 4 parameters in the  model with treatment
adjRsqManual(rsq_chi_work_withTreat_bobyqa, # 0.658
             length(PC1.NONWORK),4) 
## 3 parameters in the model without treatment
adjRsqManual(rsq_chi_work_noTreat_bobyqa, # 0.703
             length(PC1.NONWORK),3)






## THEN DO SONORAN DATA

## gamma distributions require non-negative data, so
## adjust the PC1 values so that they are non-negative
PC1.NONZERO = son$PC1.SON +2
PC1.NONWORK = son_work$PC1.SON +2

PRE.NONZERO = son$PRE.PC1.SON +2
PRE.NONWORK = son_work$PRE.PC1.SON +2

## test two optimization algorithms on the Treatment-Included dataset
## model in manuscript:
## Aggression(PC1) ~ Treatment + Pre-Treatment Aggression + Site(Random Effect) + Stimulus Set(Random Effect)
## equivalent model below:
## PC1.NONZERO ~ Type + PRE.NONZERO + (1 | Point) + (1 | Song)

## model 1A: full dataset, with treatment, L-BFGS-B model
son_full_withTreat_lbfgsb = glmer(PC1.NONZERO ~ Type + PRE.NONZERO + (1 | Point) + (1 | Song),
                                  data = son, family = Gamma(link = "log"),
                                  control = glmerControl(optimizer = "optimx", calc.derivs = FALSE,
                                                         optCtrl = list(method = "L-BFGS-B", starttests = FALSE, kkt = FALSE)))  
## model 1B: full dataset, no treatment, L-BFGS-B model
son_full_noTreat_lbfgsb = glmer(PC1.NONZERO ~ PRE.NONZERO + (1 | Point) + (1 | Song),
                                data = son, family = Gamma(link = "log"),
                                control = glmerControl(optimizer = "optimx", calc.derivs = FALSE,
                                                       optCtrl = list(method = "L-BFGS-B", starttests = FALSE, kkt = FALSE)))  

## model 2A: full dataset, with treatment, BOBYQA model
son_full_withTreat_bobyqa = glmer(PC1.NONZERO ~ Type + PRE.NONZERO + (1 | Point) + (1 | Song),
                                  data = son, family = Gamma(link = "log"),
                                  control = glmerControl(optimizer = "optimx", calc.derivs = FALSE,
                                                         optCtrl = list(method = "bobyqa", starttests = FALSE, kkt = FALSE)))  
## model 2B: full dataset, no treatment, BOBYQA model
son_full_noTreat_bobyqa = glmer(PC1.NONZERO ~ PRE.NONZERO + (1 | Point) + (1 | Song),
                                data = son, family = Gamma(link = "log"),
                                control = glmerControl(optimizer = "optimx", calc.derivs = FALSE,
                                                       optCtrl = list(method = "bobyqa", starttests = FALSE, kkt = FALSE)))  

## model 3A: detections-only dataset, with treatment, L-BFGS-B model
son_work_withTreat_lbfgsb = glmer(PC1.NONWORK ~ Type + PRE.NONWORK + (1 | Point) + (1 | Song),
                                  data = son_work, family = Gamma(link = "log"),
                                  control = glmerControl(optimizer = "optimx", calc.derivs = FALSE,
                                                         optCtrl = list(method = "L-BFGS-B", starttests = FALSE, kkt = FALSE)))  
## model 3B: detections-only dataset, no treatment, L-BFGS-B model
son_work_noTreat_lbfgsb = glmer(PC1.NONWORK ~ PRE.NONWORK + (1 | Point) + (1 | Song),
                                data = son_work, family = Gamma(link = "log"),
                                control = glmerControl(optimizer = "optimx", calc.derivs = FALSE,
                                                       optCtrl = list(method = "L-BFGS-B", starttests = FALSE, kkt = FALSE)))  

## model 4A: detections-only dataset, with treatment, BOBYQA model
son_work_withTreat_bobyqa = glmer(PC1.NONWORK ~ Type + PRE.NONWORK + (1 | Point) + (1 | Song),
                                  data = son_work, family = Gamma(link = "log"),
                                  control = glmerControl(optimizer = "optimx", calc.derivs = FALSE,
                                                         optCtrl = list(method = "bobyqa", starttests = FALSE, kkt = FALSE)))  
## model 4B: detections-only dataset, with treatment, BOBYQA model
son_work_noTreat_bobyqa = glmer(PC1.NONWORK ~ PRE.NONWORK + (1 | Point) + (1 | Song),
                                data = son_work, family = Gamma(link = "log"),
                                control = glmerControl(optimizer = "optimx", calc.derivs = FALSE,
                                                       optCtrl = list(method = "bobyqa", starttests = FALSE, kkt = FALSE)))  

## get AICc of each model 
AICc(son_full_withTreat_lbfgsb) #711.39
AICc(son_full_noTreat_lbfgsb) #722.77
AICc(son_full_withTreat_bobyqa) #711.39
AICc(son_full_noTreat_bobyqa) #722.77

AICc(son_work_withTreat_lbfgsb) #578.99
AICc(son_work_noTreat_lbfgsb) #588.96
AICc(son_work_withTreat_bobyqa) #578.99
AICc(son_work_noTreat_bobyqa) #588.96

## Note: L-BFGS-B model and BOBYQA model produce identical results, so 
## only use BOBYQA from now on (model 2 and model 4)

summary(son_work_withTreat_bobyqa) {
  "Generalized linear mixed model fit by maximum likelihood (Laplace Approximation) ['glmerMod']
 Family: Gamma  ( log )
  Formula: PC1.NONWORK ~ Type + PRE.NONWORK + (1 | Point) + (1 | Song)
  Data: son_work
  Control: glmerControl(optimizer = "optimx", calc.derivs = FALSE, optCtrl = list(method = "bobyqa",  
  starttests = FALSE, kkt = FALSE))
  
  AIC      BIC   logLik deviance df.resid 
  578.3    604.8   -281.1    562.3      196 
  
  Scaled residuals: 
  Min      1Q  Median      3Q     Max 
  -1.4147 -0.6780 -0.3704  0.6204  3.2809 
  
  Random effects:
  Groups   Name        Variance Std.Dev.
  Point    (Intercept) 0.04228  0.2056  
  Song     (Intercept) 0.03508  0.1873  
  Residual             0.33096  0.5753  
  Number of obs: 204, groups:  Point, 51; Song, 17
  
  Fixed effects:
  Estimate Std. Error t value Pr(>|z|)    
  (Intercept)  0.002143   0.143849   0.015    0.988    
  TypeCW.S    -0.182522   0.235030  -0.777    0.437    
  TypeNM      -0.077673   0.163035  -0.476    0.634    
  TypePT.S     0.911355   0.164676   5.534 3.13e-08 ***
  PRE.NONWORK  0.391421   0.077472   5.052 4.36e-07 ***
  ---
  Signif. codes:  0 ‘***’ 0.001 ‘**’ 0.01 ‘*’ 0.05 ‘.’ 0.1 ‘ ’ 1
  
  Correlation of Fixed Effects:
  (Intr) TyCW.S TypeNM TyPT.S
  TypeCW.S    -0.380                     
  TypeNM      -0.583  0.354              
  TypePT.S    -0.585  0.350  0.506       
  PRE.NONWORK -0.551 -0.040  0.007  0.021"}

summary(son_work_withTreat_bobyqa) {
  "Generalized linear mixed model fit by maximum likelihood (Laplace Approximation) ['glmerMod']
 Family: Gamma  ( log )
  Formula: PC1.NONWORK ~ Type + PRE.NONWORK + (1 | Point) + (1 | Song)
  Data: son_work
  Control: glmerControl(optimizer = "optimx", calc.derivs = FALSE, optCtrl = list(method = "bobyqa",  
  starttests = FALSE, kkt = FALSE))
  
  AIC      BIC   logLik deviance df.resid 
  578.3    604.8   -281.1    562.3      196 
  
  Scaled residuals: 
  Min      1Q  Median      3Q     Max 
  -1.4147 -0.6780 -0.3704  0.6204  3.2809 
  
  Random effects:
  Groups   Name        Variance Std.Dev.
  Point    (Intercept) 0.04228  0.2056  
  Song     (Intercept) 0.03508  0.1873  
  Residual             0.33096  0.5753  
  Number of obs: 204, groups:  Point, 51; Song, 17
  
  Fixed effects:
  Estimate Std. Error t value Pr(>|z|)    
  (Intercept)  0.002143   0.143849   0.015    0.988    
  TypeCW.S    -0.182522   0.235030  -0.777    0.437    
  TypeNM      -0.077673   0.163035  -0.476    0.634    
  TypePT.S     0.911355   0.164676   5.534 3.13e-08 ***
  PRE.NONWORK  0.391421   0.077472   5.052 4.36e-07 ***
  ---
  Signif. codes:  0 ‘***’ 0.001 ‘**’ 0.01 ‘*’ 0.05 ‘.’ 0.1 ‘ ’ 1
  
  Correlation of Fixed Effects:
  (Intr) TyCW.S TypeNM TyPT.S
  TypeCW.S    -0.380                     
  TypeNM      -0.583  0.354              
  TypePT.S    -0.585  0.350  0.506       
  PRE.NONWORK -0.551 -0.040  0.007  0.021"
}

## perform an ANOVA on the full models 
Anova(son_full_withTreat_bobyqa) {
  'Analysis of Deviance Table (Type II Wald chisquare tests)

  Response: PC1.NONZERO
  Chisq Df Pr(>Chisq)    
  Type        58.069  3  1.519e-12 ***
  PRE.NONZERO 31.888  1  1.633e-08 ***
  ---
  Signif. codes:  0 ‘***’ 0.001 ‘**’ 0.01 ‘*’ 0.05 ‘.’ 0.1 ‘ ’ 1' }

Anova(son_work_withTreat_bobyqa) {
  'Analysis of Deviance Table (Type II Wald chisquare tests)

  Response: PC1.NONWORK
  Chisq Df Pr(>Chisq)    
  Type        49.538  3  1.002e-10 ***
  PRE.NONWORK 25.527  1  4.362e-07 ***
  ---
  Signif. codes:  0 ‘***’ 0.001 ‘**’ 0.01 ‘*’ 0.05 ‘.’ 0.1 ‘ ’ 1' }

## calculate R-squared and adjusted R-squared for those models 

rsq_son_full_withTreat_bobyqa = (manualRsq(son_full_withTreat_bobyqa,
                                           PC1.NONZERO)) ## 0.905
rsq_son_full_noTreat_bobyqa = (manualRsq(son_full_noTreat_bobyqa,
                                         PC1.NONZERO)) ## 0.905

rsq_son_work_withTreat_bobyqa = (manualRsq(son_work_withTreat_bobyqa,
                                           PC1.NONWORK)) ## 0.897
rsq_son_work_noTreat_bobyqa = (manualRsq(son_work_noTreat_bobyqa,
                                         PC1.NONWORK)) ## 0.899

## 4 parameters in the  model with treatment
adjRsqManual(rsq_son_full_withTreat_bobyqa, # 0.904
             length(PC1.NONZERO),4) 
## 3 parameters in the model without treatment
adjRsqManual(rsq_son_full_noTreat_bobyqa, # 0.904
             length(PC1.NONZERO),3) 

## 4 parameters in the  model with treatment
adjRsqManual(rsq_son_work_withTreat_bobyqa, # 0.895
             length(PC1.NONWORK),4) 
## 3 parameters in the model without treatment
adjRsqManual(rsq_son_work_noTreat_bobyqa, # 0.897
             length(PC1.NONWORK),3)
